package example1;

/**
 * This class represents a Car with an ID and a history log.
 * It demonstrates reference copy, shallow copy, and deep copy.
 */
public class Car {
    private int id; // Unique identifier for the car
    private StringBuffer history; // Mutable history log for the car's maintenance
    private static int total = 0; // Tracks the total number of Car instances

    public Car(int id) {
        this.id = id;
        history = new StringBuffer();
        total++;
    }

    public Car() {
        this(0);
    }

    public String toString() {
        return "Id: " + id + ", History: " + history;
    }

    public StringBuffer getHistory() {
        return new StringBuffer(history);
    }

    public void addHistory(String item) {
        history.append(item);
    }

    public static void referenceCopy(Car car1, Car car2) {
        System.out.println("Reference Copy Example");

        // car1 now references the same object as car2
        car1 = car2;

        System.out.println("Car1 (after referenceCopy): " + car1);
        System.out.println("Car2 (after referenceCopy): " + car2);

        // Modify car1 and observe changes in car2
        car1.addHistory(", Add maintenance to Car1 (via reference copy)");
        System.out.println("Car1 (after modification): " + car1);
        System.out.println("Car2 (after Car1 modification): " + car2);

        System.out.println("End Reference Copy Example\n");
    }

    public static void shallowCopy(Car car1, Car car2) {
        System.out.println("Shallow Copy Example");

        car1.id = car2.id; // Copy primitive field
        car1.history = car2.history; // Share mutable object

        System.out.println("Car1 (after shallowCopy): " + car1);
        System.out.println("Car2 (after shallowCopy): " + car2);

        // Modify car1 and observe changes in car2
        car1.addHistory(", Add maintenance to Car1 (via shallow copy)");
        System.out.println("Car1 (after modification): " + car1);
        System.out.println("Car2 (after Car1 modification): " + car2);

        System.out.println("End Shallow Copy Example\n");
    }

    public static void deepCopy(Car car1, Car car2) {
        System.out.println("Deep Copy Example");

        car1.id = car2.id; // Copy primitive field
        car1.history = new StringBuffer(car2.history); // Create a new copy of the mutable object

        System.out.println("Car1 (after deepCopy): " + car1);
        System.out.println("Car2 (after deepCopy): " + car2);

        // Modify car1 and observe changes in car2
        car1.addHistory(", Add maintenance to Car1 (via deep copy)");
        System.out.println("Car1 (after modification): " + car1);
        System.out.println("Car2 (after Car1 modification): " + car2);

        System.out.println("End Deep Copy Example\n");
    }

    public static int getTotal() {
        return total;
    }

    public static void main(String[] args) {
        // Initialize two Car objects with different IDs and histories
        Car car1 = new Car(10), car2 = new Car(20);
        car1.addHistory("Oil change");
        car1.addHistory(", Fix door");
        car2.addHistory("Paint car");

        // Print initial states
        System.out.println("Initial Cars:");
        System.out.println("Car1: " + car1);
        System.out.println("Car2: " + car2 + "\n");

        // Demonstrate the three copy methods with modifications
        System.out.println("Demonstrating Copy Methods:");
        referenceCopy(car1, car2);

        // Reset car1 to demonstrate subsequent methods clearly
        car1 = new Car(10);
        car1.addHistory("Oil change");
        car1.addHistory(", Fix door");

        shallowCopy(car1, car2);

        // Reset car1 again for the deep copy demonstration
        car1 = new Car(10);
        car1.addHistory("Oil change");
        car1.addHistory(", Fix door");

        deepCopy(car1, car2);

        // Print the total number of cars created
        System.out.println("Total Cars Created: " + getTotal());
    }
}
